import os
from jproperties import Properties

class Params(object):

    def __init__(self, name="params", **key_vals):
        self.name = name
        self.key_vals = key_vals
        self.properties = Properties()

    def _get_items(self):
        return self.key_vals.items()

    def _add_items(self):
        for name, value in self._get_items():
            self.properties[name] = value

    def _write(self, folder):
        name = "{}.properties".format(self.name)
        path = os.path.join(folder, name)
        with open(path, "wb") as f:
            self.properties.store(f, encoding="utf-8", initial_comments=name, timestamp=False)


class Env(Params):

    def _get_items(self):
        return [("env.{}".format(name), value) for name, value in self.key_vals.items()]


class ParamsJoin:

    def __init__(self, param_name="custom.envs", separator=" "):
        self.param_name = param_name
        self.separator = separator
        self.properties = Properties()
        self.params_class = []

    def __call__(self, *params_class):
        self.params_class = params_class
        return self

    def _add_items(self):
        param_props = []
        for param_class in self.params_class:
            for name, value in param_class._get_items():
                param_props.append(value)
        self.properties[self.param_name] = self.separator.join(param_props)

    def _write(self, folder):
        name = "{}.properties".format("_".join([param_class.name for param_class in self.params_class]))
        path = os.path.join(folder, name)
        with open(path, "wb") as f:
            self.properties.store(f, encoding="utf-8", initial_comments=name, timestamp=False)



import copy

class Project(object):

    def __init__(self, folder, *jobtypes):
        self.jobtypes = jobtypes
        self.folder = folder
        self.params = []
        self.version = 1

    def is_v2(self):
        self.version = 2
        return copy.deepcopy(self)

    def is_v1(self):
        self.version = 1
        return copy.deepcopy(self)

    def with_params(self, *paramtypes):
        self.params = paramtypes
        return copy.deepcopy(self)

    def write(self):

        for param in self.params:
            param._add_items()
            param._write(self.folder)

        for jobtype in self.jobtypes:
            jobtype._add_items()
            jobtype._write(self.folder)
        
        if self.version == 2:
            with open('flow20.project', 'w') as project:
                project.write('azkaban-flow-version: 2.0')


#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import shutil, tempfile
from unittest import TestCase
from auror_core import Project
from auror_core.job import Job, Command
from auror_core.params import Params, Env, ParamsJoin

params = Params(
    "params",
    retries="2",
    **{
        "retry.backoff": "30000",
        "failure.emails": "email@gmail.com"
    }
)

envs = Env(
    "some_envs",
    SOMETHING_KEY = "xxxxxxxx",
    SPARK_MASTER="yarn"
)

job1_command = Job() \
    .as_type(Command) \
    .with_name("job1_command") \
    .with_command("${python} -c 'from test import test_method; test_method()'")


class TestCaseWithNecessaryConditions(TestCase):

    def setUp(self):
        self.test_dir = tempfile.mkdtemp()

    def tearDown(self):
        shutil.rmtree(self.test_dir, ignore_errors=True)

    def list_files(self):
        return os.listdir(self.test_dir)

    def open_file(self, name):
        return open(os.path.join(self.test_dir, name))


class ParamsPropertiesFileTest(TestCaseWithNecessaryConditions):

    def test_check_if_directory_created_has_params_file_only(self):
    	project = Project(self.test_dir)
        project.with_params(params).write()

        tmp_dir = self.list_files()

        self.assertEqual(1, len(tmp_dir))
        self.assertTrue("params.properties" in tmp_dir)

    def test_check_the_content_written_in_params_properties_file(self):
        project = Project(self.test_dir)
        project.with_params(params).write()

        f = self.open_file("params.properties")
        expected = "#params.properties\nfailure.emails=email@gmail.com\nretries=2\nretry.backoff=30000\n"

        self.assertEqual(expected, f.read())


class EnvPropertiesFileTest(TestCaseWithNecessaryConditions):

    def test_check_if_directory_created_has_env_file_only(self):
        project = Project(self.test_dir)
        project.with_params(ParamsJoin()(envs)).write()

        tmp_dir = self.list_files()

        self.assertEqual(1, len(tmp_dir))
        self.assertTrue("some_envs.properties" in tmp_dir)

    def test_check_the_content_written_in_envs_properties(self):
        project = Project(self.test_dir)
        project.with_params(ParamsJoin()(envs)).write()

        f = self.open_file("some_envs.properties")
        expected = "#some_envs.properties\ncustom.envs=yarn xxxxxxxx\n"

        self.assertEqual(expected, f.read())


class JobTypeCommandTest(TestCaseWithNecessaryConditions):

    def test_check_if_directory_created_has_job1_command_file_only(self):
        project = Project(self.test_dir, job1_command)
        project.with_params().write()

        tmp_dir = self.list_files()

        self.assertEqual(1, len(tmp_dir))
        self.assertTrue("job1_command.job" in tmp_dir)

    def test_check_the_content_written_in_job1(self):
        project = Project(self.test_dir, job1_command)
        project.with_params().write()

        f = self.open_file("job1_command.job")
        expected = "#job1_command.job\ncommand=${python} -c 'from test import test_method; test_method()'\ntype=command\n"

        self.assertEqual(expected, f.read())



# encoding: utf-8

from setuptools import setup, find_packages

with open("version.txt") as version_file:
    version = version_file.read().strip()

setup(
    name="auror_core",
    version=version,
    description="Doing Jobs Dinamically with Azkaban",
    author="Big Data",
    author_email="bigdata@corp.globo.com",
    license='MIT',
    install_requires=[
        "jproperties",
        "pyaml"
    ],
    packages=find_packages(),
)



import os
import yaml

class Params(object):

    def __init__(self, name="params", **key_vals):
        self.name = name
        self.key_vals = key_vals
        self.properties = dict(config=dict())

    def _get_items(self):
        return self.key_vals.items()

    def _add_items(self):
        self.properties['config'] = dict(self._get_items())

    def _write(self, folder):
        name = os.path.basename(folder)
        path = "{}.flow".format(os.path.join(folder, name))
        try:
            with open(path, 'rb') as reader:
                data = yaml.load(reader)
            data.update(self.properties)
        except IOError:
            data = self.properties

        with open(path, 'wb') as writer:
            writer.write(
                yaml.dump(data, default_flow_style=False)
            )

class Env(Params):

    def _get_items(self):
        return [("env.{}".format(name), value) for name, value in self.key_vals.items()]



class ParamsJoin(Params):

    def __init__(self, param_name="custom.envs", separator=" "):
        self.param_name = param_name
        self.separator = separator
        self.properties = dict(config=dict())
        self.params_class = []

    def __call__(self, *params_class):
        self.params_class = params_class
        return self

    def _add_items(self):
        param_props = []
        for param_class in self.params_class:
            for name, value in param_class._get_items():
                param_props.append(value)
        self.properties["config"][self.param_name] = self.separator.join(param_props)